clear 
clear mata 
clear matrix 
cd "PATH WHERE DATA IS AER-2012-0141-REPLICATION/DATA"

		*** ============================= *** 
		***							      ***
		*** FIGURE 1 & 4: AUCTION PRICES  *** 
		***							      ***
		*** ============================= ***

clear 
use F4.dta 
sort weekseason season

label var weekseason "Week of the Season"
label var pauctnorm "Weekly Prices at the Auction"

		**************** 
		****************
		*** FIGURE 1 ***
		****************
		****************

twoway (line pauctnorm weekseason if season == 2) (line  pauctnorm weekseason if season == 3) (line pauctnorm weekseason if season == 4)  
xi : areg pauctnorm i.season , abs(weekseason)

capture drop ratio 
gen ratio = plarge/psmall 

		**************** 
		****************
		*** FIGURE 4 ***
		****************
		****************
		
label var ratio "Weekly Auction Price ratio between Large and Small roses"
twoway (line ratio weekseason if season == 2) (line  ratio weekseason if season == 3) (line ratio weekseason if season == 4), xline(26)  


		*** ============================= *** 
		***							      ***
		*** FIGURE 2: PRICE FLUCTUATIONS  *** 
		***							      ***
		*** ============================= ***
		
clear
use F5
label var weekseason "Week of the Season, 2006-07"
label var PD "Direct Relationships (Estimated)"
label var PA "Auctions"
twoway (line PD weekseason) (line PA weekseason), xline(26)  
 

		
		*** =============================================== *** 
		***							      					***
		*** FIGURE 3: DISTRIBUTION HIGHEST TEMPTATION WEEK  *** 
		***							      					***
		*** =============================================== ***
		
clear
use F1
keep if period == 1 & season == 3 & V1>0

gen d = (weekMT1 * 7) -4 
gen dateMT = doy(d) 
gen monthMT = month(d)
label define monthMT 1 "Jan" 2 "Feb" 3 "Mar" 4 "Apr" 5 "May" 6 "Jun" 7 "Jul" 8 "Aug" 9 "Sep" 10 "Oct" 11 "Nov" 12 "Dec"

replace dateMT = 1+floor(dateMT/30)
label var weekMT1 "Week of Maximum Temptation to Deviate"
twoway (hist weekMT1 , bin(52) frac) 

		*** ===================================== *** 
		***							      		  ***
		*** FIGURE 5: STRETCH IN MAXIMUM QUANTITY *** 
		***							      		  ***
		*** ===================================== ***
		
clear
use  F1

keep if period == 1 & season == 3 & sample == 1 & new == 0 

gen Qrel = Q1/ (weekq*1000)

twoway (kdensity Qrel) 
		
		*** ============================= *** 
		***							      ***
		*** FIGURE 6: RELATIONSHIP DEATH  *** 
		***							      ***
		*** ============================= ***
		
clear
use F6 

label var weekseason "Weeks, Season 2006/07 and 2007/08"
label var NA "Number of Relationships Ending"
label var NN "Violence Period"
label var PA "Auction Prices (Relative to Season Average)"

twoway (bar NA weekseason , yaxis(1)) (bar NN weekseason , yaxis(1)) (line PA weekseason , yaxis(2)) 

gen violence_weeks = 0 
replace violence_weeks = 1 if weekseason == 75 | weekseason == 76 

gen week = weekseason 
replace week = weekseason - 52 if weekseason > 52 

xi : areg NA PA violence_weeks i.season , abs(week)
		
		
		*** ============================= *** 
		***							      ***
		*** FIGURE 7: RELATIONSHIPS VALUE *** 
		***							      ***
		*** ============================= ***		

clear 
use F1

drop if season == 1 | season > 3
drop if period != 1 

gen lnS = ln(S1)

twoway (kdensity lnS if season == 3 & sample == 1 & new != 1 ) (kdensity lnS if season == 3 & new == 1) (kdensity lnS if season == 3 & sample == 2) 

		*** ============================= *** 
		***							      ***
		*** FIGURE 8: AGE & RELAT. VALUE *** 
		***							      ***
		*** ============================= ***		
				
clear 
use  F1

keep if season == 3 & period == 2 & new != 1 

gen lnS = ln(S1)

twoway (scatter lnS pastinter, mlabel(idrel)) (lowess lnS pastinter) 

clear
use  F1
keep if period == 2 & balance == 1 

gen lnS = ln(S1)

twoway (scatter lnS pastinter, mlabel(idrel)) (lowess lnS pastinter) 

** NB: combine both graphs by using combine command in Stata **

		*** ============================= *** 
		***							      ***
		*** FIGURE 9: RELIABILITY & CONF *** 
		***							      ***
		*** ============================= ***		

clear 
use F2 
keep if direct == 1 
twoway (scatter reliab pastinter if conf ==1 ) (lowess reliab pastinter if conf ==1 ) (scatter reliab pastinter if conf ==0 ) (lowess reliab pastinter if conf == 0)

		
		*** ============================= *** 
		***							      ***
		*** FIGURE 10: PRICES AT VIOLENCE *** 
		***							      ***
		*** ============================= ***		

clear 
use F7 
twoway (histogram price1 , bin(30) xline(0.11) xline(0.19)) (kdensity price1 ) (kdensity price0 ) if price0 <0.3 & price1 <0.3
