clear 
clear mata 
clear matrix 

cd "PATH WHERE DATA IS AER-2012-0141-REPLICATION/DATA"

**************************************************************************************************
* This do-file first creates the main tables for the paper: Table 1, 2, 3, 4, 5, 6 and 7 *********
**************************************************************************************************

		*** =============================== *** 
		***									***
		*** TABLE 1: DESCRIPTIVE STATISTICS *** 
		***									***
		*** =============================== ***
				
clear
use F1

* This keeps the baseline period of the season during which violence occured
keep if period == 1 & season == 4 & sample == 1 

		*********************************************
		*** PANEL A: RELATIONSHIP CHARACTERISTICS ***
		*********************************************
foreach x in transactions weekq price age pastinter censored  {
sum `x' 
}
*

		*************************************************************
		*** PANEL B: NUMBER OF RELATIONSHIPS PER BUYER AND SELLER ***
		*************************************************************

bys firmid : gen ccf =_n 
bys buyerid : gen ccb = _n 

bys firmid : gen NB =_N		
bys buyerid : gen NS =_N		
		
sum NB if ccf == 1 
sum NS if ccb == 1 
		
		************************************************************************
		*** PANEL C: ESTIMATED RELATIONSHIPS VALUES (SEASON BEFORE VIOLENCE) ***
		************************************************************************

clear 
use F1 

keep if period == 2 & season == 3 & sample == 1 
		
gen Srel = S1/ weekrev
gen Vrel = V1/weekrev
gen Urel = U1/weekrev 

sum Srel Urel Vrel 

		************************************************************************		
		************************************************************************
		************************************************************************

		
		*** =============================================== *** 
		***										            ***
		*** TABLE 2: BINDING INCENTIVE CONSTRAINTS (TEST 1) *** 
		***									                ***
		*** =============================================== ***

clear
use  F1

keep if period == 2  

keep if balance == 1 

foreach v in Q S P PA {
foreach x in 1 2 {
gen ln`v'`x' = ln(`v'`x')
}
}
*

foreach X in Q S P { 
qui xi: areg ln`X'1 lnPA1 lnpastinter i.season i.weekMT1 , abs(idrel) cl(firmid)
display _b[lnPA1]
display _se[lnPA1]
display e(r2_a)
test _b[lnPA1] = -1
test _b[lnPA1] = 0 
}
*

		************************************************************************		
		************************************************************************
		************************************************************************

		*** =============================================== *** 
		***										            ***
		*** TABLE 3: BINDING INCENTIVE CONSTRAINTS (TEST 1) *** 
		***									                ***
		*** =============================================== ***

xi: areg lnQ1 lnPA1 i.season lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA1] = -1
 
xi: areg lnQ1 lnPA1 i.weekMT1 lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA1] = -1 

xi: areg lnQ2 lnPA2 i.weekMT2 i.season lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA2] = -1 

xi: areg lnS1 lnPA1 i.season lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA1] = 0
 
xi: areg lnS1 lnPA1 i.weekMT1 lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA1] = 0 

xi: areg lnS2 lnPA2 i.weekMT2 i.season lnpastinter , abs(idrel) cl(firmid)
test _b[lnPA2] = 0 


		*** =============================================== *** 
		***										            ***
		*** TABLE 4: AGE AND RELATIONSHIP OUTCOMES (TEST 2) *** 
		***									                ***
		*** =============================================== ***

		**************************************
		*** CROSS-SECTION: COLUMNS 1,3,5,7 ***
		**************************************

clear 
use  F1

keep if season == 3 & period == 2 & new != 1 

gen lnQ = ln(Q1)
gen lnS = ln(S1)
gen lnV = ln(V1)
gen lnU = ln(U1)

		// PANEL A // 

foreach X in Q S U V {
xi: areg ln`X' lnpastinter i.firmid , abs(buyerid) cl(firmid )
}
*

		// PANEL B // 

replace pastinter = pastinter / 100 

foreach X in Q S U V {
xi: areg ln`X' pastinter i.firmid , abs(buyerid) cl(firmid )
}
*
		
		**************************************
		*** PANEL SPECIF.A ONLY: COLUMNS 2,4,6,8 ***
		**************************************

clear
use  F1
keep if period == 2 & balance == 1 

gen lnQ = ln(Q1)
gen lnS = ln(S1)
gen lnV = ln(V1)
gen lnU = ln(U1)

foreach X in Q S U V {
xi: areg ln`X' lnpastinter i.season , abs(idrel) cl(firmid )
}
*		

		*** =============================================== *** 
		***										            ***
		*** TABLE 5: AGE AND RELATIONSHIP OUTCOMES (TEST 2) *** 
		***									                ***
		*** =============================================== ***
		
		// Controlling for seasonality [Columns 1 & 5]

foreach X in Q S {
xi: areg ln`X' lnpastinter i.weekMT1  i.season , abs(idrel) cl(firmid)
}
*
	// Two weeks period [Columns 2 & 6]

foreach X in Q S {
capture drop lnw`X'		
gen lnw`X' = ln(`X'w2)			
xi: areg lnw`X' lnpastinter i.season , abs(idrel) cl(firmid)
}
*
			
	// Age as calendar [Columns 3 & 7]				
					
foreach X in Q S {
xi: areg ln`X' lnage i.season , abs(idrel) cl(firmid) 
}
*

	// Controlling for maximum past price [Columns 4 & 8]

foreach X in Q S {
xi: areg ln`X' lnpastinter maxPA1past i.season , abs(idrel) cl(firmid)
}
*			

		*** =============================================== *** 
		***										            ***
		*** TABLE 6: RELIABILITY AT VIOLENCE TIME  (TEST 3) *** 
		***									                ***
		*** =============================================== ***

clear 
use F2 

keep if direct == 1 
					
scalar drop _all
foreach cl in firmid buyerid idrel {
qui xi : reg reliab conf 					, cl(`cl')
display e(r2_a)
scalar def e`cl' = _se[conf]
}
*
											***************************************
											*************** COLUMN 1 ************** 
											***************************************

display "REGRESSION 1"
display _b[conf]
display (efirmid + ebuyerid - eidrel) 
display _b[conf]/(efirmid + ebuyerid - eidrel) 
display e(r2_a)

											***************************************
											*************** COLUMN 2 ************** 
											***************************************

scalar drop _all
foreach cl in firmid buyerid idrel {
qui xi : reg reliab conf 	censored price weekq land fair_trade firm_start kfc_member indian foreign i.buyerid, cl(`cl')
scalar def e`cl' = _se[conf]
}
*
display "REGRESSION 2"
display _b[conf]
display (efirmid + ebuyerid - eidrel) 
display _b[conf]/(efirmid + ebuyerid - eidrel) 
display e(r2_a)

											*********************************************
											*************** COLUMN 3 AND 4 ************** 
											*********************************************


replace pastinter = pastinter / 100											
gen pastinter2 = pastinter^2
											
scalar drop _all
foreach c in 0 1 {
foreach cl in firmid buyerid idrel {
qui xi : areg reliab pastinter pastinter2 censored price weekq i.buyerid if conf == `c', abs(firmid) cl(`cl')
scalar def e`cl' = _se[pastinter]
scalar def ee`cl' = _se[pastinter2]
}
*
display "REGRESSION `c'"
display _b[pastinter]
display (efirmid + ebuyerid - eidrel) 
display _b[pastinter]/(efirmid + ebuyerid - eidrel) 

display _b[pastinter2]
display (eefirmid + eebuyerid - eeidrel) 
display _b[pastinter2]/(eefirmid + eebuyerid - eeidrel) 
display e(r2_a)
}
*

		*** =============================================== *** 
		***										            ***
		*** TABLE 7: RELIABILITY AT VIOLENCE TIME  (TEST 3) *** 
		***									                ***
		*** =============================================== ***

		*******************************************
		*** AUCTIONS VS DIRECT: COLUMNS 1 and 2 ***
		*******************************************
clear 
use F2 
		
xi: reg reliab i.conf*direct, cl(firmid)
xi: areg reliab i.conf*direct , abs(firmid) cl(firmid) 


		*************************************
		*** WORKERS LOST: COLUMNS 3 and 4 ***
		*************************************
clear 
use F3

gen A = 0 
replace A = 1 if auction_pc > 90.01

gen B = 0 
replace B = 1 if auction_pc < 90.01 & auction_pc > 10.01

xi: reg lost A B , robust

replace kiku = 0 if kiku == . 

xi: reg lost A B land firm_start fair_trade kfc_member indian foreign connected temp fem house kiku ethnic_missin i.location , robust

twoway (lowess lost aucion_pc)
