function [ out_logLik_GLR ] = logLik_GLR(  betahat_long, betahat_short, sig_biv, r_d, CV_struct )
%Computes generalized log likelihood under H1 and H0
%   size(bhat) = (2, )
%   long coef. in column 1, short coef. in column 2
%   sig is covariance matrix of bhat
%   alpha = .05;
CV_vals = CV_struct.CV_vals;
angle_vec = CV_struct.angle_vec;
lograd_vec = CV_struct.lograd_vec;

sig_L = sqrt(sig_biv.sigL2);
sig_LS = sig_biv.sigLS;
sig_S = sqrt(sig_biv.sigS2);

rho_LS = sig_LS/(sig_S*sig_L);

tauhat_b = betahat_long./sig_L;
a1 = (rho_LS^(-2) - 1)^(-1/2)*(sig_L^(-1));
a2 = -(rho_LS^(-2) - 1)^(-1/2)*(sig_S^(-1)*rho_LS^(-1));
tauhat_theta = a1*betahat_long + a2*betahat_short;

psi_b = sig_L*(rho_LS^(-2) - 1)^(-1/2)*(sig_L^(-1)-rho_LS^(-1)*sig_S^(-1));
max_tau_g = abs((rho_LS^(-2) - 1)^(-1/2)*rho_LS^(-1)*sig_S^(-1)*r_d);

angle = atan(max_tau_g./abs(psi_b))/(.5*pi);
lograd = .5*(.5*log(abs(psi_b).^2+max_tau_g.^2)/log(200)+1);

[CV_ind_angle1,CV_ind_angle2] = findnearest(angle',angle_vec');
[CV_ind_lograd1,CV_ind_lograd2] = findnearest(lograd',lograd_vec');

%Find rejection profile
out_logLik_GLR = zeros(length(betahat_long),1);

ii=1;
    temp = logLik_GLR_norm(tauhat_b(ii),tauhat_theta(ii),psi_b(ii),max_tau_g(ii));
    wgt_angle = (angle(ii)-angle_vec(CV_ind_angle2(ii)))/...
        (angle_vec(CV_ind_angle1(ii))-angle_vec(CV_ind_angle2(ii)));
    wgt_lograd = (lograd(ii)-lograd_vec(CV_ind_lograd2(ii)))/...
        (lograd_vec(CV_ind_lograd1(ii))-lograd_vec(CV_ind_lograd2(ii)));
    wgt_angle = min(abs(wgt_angle),1);
    wgt_lograd = min(abs(wgt_lograd),1);   
    CV_temp = wgt_angle*wgt_lograd*CV_vals(CV_ind_lograd1(ii),CV_ind_angle1(ii)) +...
         (1-wgt_angle)*wgt_lograd*CV_vals(CV_ind_lograd1(ii),CV_ind_angle2(ii)) +...
         wgt_angle*(1-wgt_lograd)*CV_vals(CV_ind_lograd2(ii),CV_ind_angle1(ii)) +...
         (1-wgt_angle)*(1-wgt_lograd)*CV_vals(CV_ind_lograd2(ii),CV_ind_angle2(ii));
    out_logLik_GLR(ii) = (temp>CV_temp);
end