clear all
directory_home = 'Insert folder location';
code = fullfile(directory_home,'\Bivariate');

Examples = {'\Stata\Macchiavello\Bivariatedata1.xlsx',...
            '\Stata\Levitt_Donohue\Bivariatedata3.xlsx',...
            '\Stata\Imbens\Bivariatedata2.xlsx'}';
alphas = [.05,.05,.05];

cd(code)        


for idata = [3,2,1]
    
    idata
    data = Examples{idata};
    data = fullfile(directory_home,data);
    location_CV = fullfile(code,'\CV_vals.mat');
    load(location_CV)

    %% Load data
    format long
    ds = dataset('XLSFile',data,'Sheet','Design');    
    % Design
    varnames = ds.Properties.VarNames(:);
    Y = double(ds.Y);
    X = double(ds.X);
    Q_ind = ~cellfun('isempty',strfind(varnames,'Q'));
    Q = double(ds(:,Q_ind));
    KQ = size(Q,2);
    Z_ind = ~cellfun('isempty',strfind(varnames,'Z'));
    Z = double(ds(:,Z_ind));
    XQ = [X,Q];
    lambda = ((XQ'*XQ)\(XQ'*Z));
    lambda = lambda(1,:)';
    
    MqY = Y - Q*((Q'*Q)\(Q'*Y));
    MqX = X - Q*((Q'*Q)\(Q'*X));
    MqZ = Z - Q*((Q'*Q)\(Q'*Z));    
    MxqY = Y - [X,Q]*(([X,Q]'*[X,Q])\([X,Q]'*Y));
    
    N = size(Y,1);
    
    %% Errors from short regression
    % Clustered standard errors
    cluster_groups = double(ds(:,'clustervar'));
    cluster_groups_unique = unique(cluster_groups);
    P = length(cluster_groups_unique);
    np = struct;
    for ii=1:P
        np(ii).idx = find(cluster_groups == cluster_groups_unique(ii));
    end
    ehat_short = Y - XQ*((XQ'*XQ)\(XQ'*Y));
    sighat_e = zeros(N,N);
    for ii=1:P
        tempind = np(ii).idx;
        temp = ehat_short(tempind)*ehat_short(tempind)';
        sighat_e(tempind,tempind) = (temp + temp')/2;
    end
    QZ = [Q,Z];
    XQZ = [X,Q,Z];
    DLong = (XQZ'*XQZ)\(XQZ');
    DLong = DLong(1,:);
    DShort = (XQ'*XQ)\(XQ');
    DShort = DShort(1,:);
    D = [DLong; DShort];    
    clear Dsig
    Dsig = D;
    VarLS = Dsig*sighat_e*Dsig';
    sig_biv.sigL2 = VarLS(1,1);
    sig_biv.sigLS = VarLS(1,2);
    sig_biv.sigS2 = VarLS(2,2);
    
    if (idata == 3)
        Mxqz = eye(N) - XQZ*((XQZ'*XQZ)\(XQZ'));
        PzMqX = projection(MqX,MqZ);
        MqZalt = MqZ - projection(MqZ,PzMqX);
        MqZalt = orth(MqZalt);    
        Mlsqz = [Q,MqZalt];
        Mlsqz = eye(N) - Mlsqz*((Mlsqz'*Mlsqz)\(Mlsqz'));

        check = sort(abs(eig(Mxqz.^2)));
        check(1:13)
        check = sort(abs(eig(Mlsqz.^2)));
        check(1:13)      
        ehat_long = Y - projection(Y,XQZ);

        sig_biv.sigL2 = (ehat_long.^2)'*((Mlsqz.^2)\((DLong.^2)'));
        sig_biv.sigS2 = (ehat_long.^2)'*((Mlsqz.^2)\((DShort.^2)'));    
        sig_biv.sigLS = (ehat_long.^2)'*((Mlsqz.^2)\((DLong.*DShort)')); 
    end    
    
    % Coefficients
    coef_biv.betaL = DLong*Y;
    coef_biv.betaS = DShort*Y;    
    
    % QC
    QC = readtable(data,'Sheet','QC');
    QC = QC.Value;
    assert(abs((QC(1) - coef_biv.betaL)/QC(1)) < 10^(-3))
    assert(abs((QC(2) - coef_biv.betaS)/QC(2)) < 10^(-3))
%     coef_biv.betaS/sqrt(sig_biv.sigS2)
    % Long t-stat
    tstatL = coef_biv.betaL/sqrt(sig_biv.sigL2)

    %%

    R2_grid = 0:.00001:.35;
    Test = zeros(length(R2_grid),1);

    r0 = norm(MqY)*sqrt(MqX'*MqZ*((MqZ'*MqZ)\(MqZ'*MqX)) / (MqX'*MqX*(MqX'*MqX)));
    alpha = alphas(idata);
    ii = 1*(alpha==.01)+2*(alpha==.05)+3*(alpha==.1);
    CV_struct = CV(ii);    
    
    iTest = 1;

    for iTest = 1:length(R2_grid)
        R2_bound = R2_grid(iTest);
        r_d = r0*sqrt(R2_bound);
        Test(iTest) = logLik_GLR(coef_biv.betaL, coef_biv.betaS, sig_biv, r_d, CV_struct);
    end
    Test1 = [Test(2:end);NaN];
    Testm1 = [NaN;Test(1:end-1)];    
    Cutoff = [Test((Test ~= Test1) | (Test ~= Testm1)), R2_grid((Test ~= Test1) | (Test ~= Testm1))'];
    Cutoffkappa = [Test((Test ~= Test1) | (Test ~= Testm1)), R2_grid((Test ~= Test1) | (Test ~= Testm1))'];
    Cutoffkappa(:,2) = Cutoffkappa(:,2)*norm(MqY)^2
    sheet = strcat('Kappabound');
    xlswrite(data,Cutoffkappa,sheet)
    sheet = string(idata);
    xlswrite(fullfile(directory_home,'\Empirical.xlsx'),[Cutoffkappa(2,2);sqrt(Cutoffkappa(2,2)/N);Cutoffkappa(2,2)/(norm(MxqY)^2)],sheet)
    
    
    Statistics = dataset('XLSFile',data,'Sheet','Statistic');
    Statistics.Value(strcmp(Statistics.Statistic,'t-statistic long')) = coef_biv.betaL/sqrt(sig_biv.sigL2);
    Statistics.Value(strcmp(Statistics.Statistic,'t-statistic short (Matlab)')) = coef_biv.betaS/sqrt(sig_biv.sigS2);
    Statistics.Value(strcmp(Statistics.Statistic,'Correlation between long and short')) = sig_biv.sigLS / sqrt(sig_biv.sigL2 * sig_biv.sigS2);
    tmp = cell2dataset({'Statistic','Value';'(Matlab) Betahat long',coef_biv.betaL});
    Statistics = [Statistics; tmp];    
    tmp = cell2dataset({'Statistic','Value';'(Matlab) Betahat short',coef_biv.betaS});
    Statistics = [Statistics; tmp];      
    tmp = cell2dataset({'Statistic','Value';'(Matlab) norm(MqY)^2',norm(MqY)^2});
    Statistics = [Statistics; tmp];
    tmp = cell2dataset({'Statistic','Value';'(Matlab) norm(MxqY)^2',norm(MxqY)^2});
    Statistics = [Statistics; tmp];
    tmp = cell2dataset({'Statistic','Value';'(Matlab) norm(MqX)^2',norm(MqX)^2});
    Statistics = [Statistics; tmp];
    tmp = cell2dataset({'Statistic','Value';'(Matlab) Var(Betahat long)',sig_biv.sigL2});
    Statistics = [Statistics; tmp];    
    tmp = cell2dataset({'Statistic','Value';'(Matlab) Cov(Betahat long, Betahat short)',sig_biv.sigLS});
    Statistics = [Statistics; tmp];      
    tmp = cell2dataset({'Statistic','Value';'(Matlab) Var(Betahat short)',sig_biv.sigS2});
    Statistics = [Statistics; tmp];  
    tmp = cell2dataset({'Statistic','Value';'(Matlab) R-squared of Z on X given Q', (MqX'*MqZ*((MqZ'*MqZ)\(MqZ'*MqX)))/(MqX'*MqX)});
    Statistics = [Statistics; tmp];    
    tmp = cell2dataset({'Statistic','Value';'(Matlab) R-squared of Z on Y given Q', (MqY'*MqZ*((MqZ'*MqZ)\(MqZ'*MqY)))/(MqY'*MqY)});
    Statistics = [Statistics; tmp]; 
    
    XQ = [X,Q];
    MxqZ = Z - XQ*((XQ'*XQ)\(XQ'*Z));
    Zcoef = (MxqZ'*MxqZ)\(MxqZ'*Y);    
    tmp = cell2dataset({'Statistic','Value';'(Matlab) norm(MqZ*Long coefficient)^2',norm(MqZ*Zcoef)^2});
    Statistics = [Statistics; tmp]; 
    
    sheet = strcat('Matlab');
    export(Statistics,'XLSfile',data,'Sheet',sheet);
    clear Statistics
    
end