clear all
directory_home = 'Insert location';
cd(directory_home)

addpath(fullfile(directory_home,'\LASSO_code\QJE'));
code = fullfile(directory_home,'\Bivariate');

Examples = {'\Stata\Macchiavello\Bivariatedata1.xlsx'}';
alphas = [.05];

idata = 1;


    data = Examples{idata};
    data = fullfile(directory_home,data);
    location_CV = fullfile(code,'\CV_vals.mat');
    load(location_CV)

    %% Load data
    format long
    ds = dataset('XLSFile',data,'Sheet','Design');    
    % Design
    varnames = ds.Properties.VarNames(:);
    Y = double(ds.Y);
    X = double(ds.X);
    Q_ind = ~cellfun('isempty',strfind(varnames,'Q'));
    Q = double(ds(:,Q_ind));
    KQ = size(Q,2);
    Z_ind = ~cellfun('isempty',strfind(varnames,'Z'));
    Z = double(ds(:,Z_ind));
    N = size(Y,1);
    Mq = eye(N) - Q*((Q'*Q)\(Q'));
    MqY = Mq*Y;
    MqX = Mq*X;
    MqZ = Mq*Z;
    
    QZ = [Q,Z];
    
    %% LPDS

    cluster_groups = double(ds(:,'clustervar'));
    cluster_groups_unique = unique(cluster_groups);
    n_in_group = zeros(size(cluster_groups_unique,1),1);
    for grval = cluster_groups_unique'
        n_in_grval = length(cluster_groups(cluster_groups == grval));
        n_in_group(grval) = n_in_grval;
    end
    DT = max(n_in_group);
 
    lambda0 = [0;10];
    lambda1 = [];
    stop = 0;
    

    maxIter = 100;
    

    for ii = 1:100
    	beta0 = zeros(length(lambda0),1);
    	SE0 = zeros(length(lambda0),1);
    	IndUnionV0 = zeros(length(lambda0),1);

    	for jj = 1:length(lambda0)
            StV = QZ.*(X*ones(1,size(QZ,2)));
            UpsV = sqrt(sum(StV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            lambda = 1.1*2*sqrt(DT*N)*norminv(1-.05/(2*size(QZ,2)))*lambda0(jj);
            PInitV = LassoShooting2(QZ, X , lambda, UpsV, 'Verbose', 0);
            IndInitV = abs(PInitV) > 0;
            ZV = QZ(:,IndInitV);
            RefResidV = X-QZ*PInitV;
            efsV = X-ZV*((ZV'*ZV)\(ZV'*X)) ;
            IndRefV = IndInitV;

            kk = 1;
            StRefV = QZ.*(RefResidV*ones(1,size(QZ,2)));
            UpsRefV = sqrt(sum(StRefV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            while norm(UpsRefV-UpsV) > 1e-4 && kk < maxIter,
                disp(kk);
                PRefV = LassoShooting2(QZ, X , lambda, UpsRefV , 'Verbose', 0);
                IndRefV = abs(PRefV) > 0;
                ZV = QZ(:,IndRefV);
                bfsV = ((ZV'*ZV)\(ZV'*X));
                efsV = X-ZV*bfsV;
                UpsV = UpsRefV;
                StRefV = QZ.*((X-QZ*PRefV)*ones(1,size(QZ,2)));
                UpsRefV = sqrt(sum(StRefV.^2)/(DT*N));    
                kk = kk+1;
            end
            sfsV = cluster_se(ZV,efsV,inv(ZV'*ZV),cluster_groups);

            % Structural equation
            StyV = QZ.*(Y*ones(1,size(QZ,2)));
            UpsyV = sqrt(sum(StyV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            PInityV = LassoShooting2(QZ, Y , lambda, UpsyV , 'Verbose', 0);
            IndInityV = abs(PInityV) > 0;
            ZyV = QZ(:,IndInityV);
            RefResidyV = Y-QZ*PInityV;
            efsyV = Y-ZyV*((ZyV'*ZyV)\(ZyV'*Y)) ;
            IndRefyV = IndInityV;

            kk = 1;
            StRefyV = QZ.*(RefResidyV*ones(1,size(QZ,2)));
            UpsRefyV = sqrt(sum(StRefyV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            while norm(UpsRefyV-UpsyV) > 1e-4 && kk < maxIter,
                disp(kk);
                PRefyV = LassoShooting2(QZ, Y , lambda, UpsRefyV, 'Verbose', 0);
                IndRefyV = abs(PRefyV) > 0;
                ZyV = QZ(:,IndRefyV);
                bfsyV = ((ZyV'*ZyV)\(ZyV'*Y));
                efsyV = Y-ZyV*bfsyV;
                UpsyV = UpsRefyV;
                StRefyV = QZ.*((Y-QZ*PRefyV)*ones(1,size(QZ,2)));
                UpsRefyV = sqrt(sum(StRefyV.^2)/(DT*N));    
                kk = kk+1;
            end
            sfsyV = cluster_se(ZyV,efsyV,inv(ZyV'*ZyV),cluster_groups);

            IndUnionV = max([IndRefyV IndRefV],[],2);
            IndUnionV(1:size(Q,2)) = 1;
            ZUnionV = QZ(:,IndUnionV);

            R2_1V = sum((ZUnionV*(ZUnionV\X)).^2)/(sum(X.^2));
            R2_2V = sum((ZUnionV*(ZUnionV\Y)).^2)/(sum(Y.^2)); 

            QZLASSO = [X ZUnionV];
            bLASSO = (QZLASSO'*QZLASSO)\(QZLASSO'*Y);
            eLASSO = Y - QZLASSO*bLASSO;
            ResLASSO = cluster_se(QZLASSO,eLASSO,inv(QZLASSO'*QZLASSO),cluster_groups);

           	beta0(jj) = bLASSO(1);
        	SE0(jj) = ResLASSO(1);
        	IndUnionV0(jj) = sum(IndUnionV)
    	end
    	M0 = [lambda0, beta0, SE0, IndUnionV0];
    	lambda1 = [lambda1; M0];
    	lambda1 = sortrows(lambda1);
        lambda0 = zeros(size(lambda1,1)-1,2);
        for jj = 1:size(lambda0,1)
            lambda0(jj,1) = (lambda1(jj+1,1) + lambda1(jj,1))/2;
            lambda0(jj,2) = (abs(lambda1(jj+1,4) - lambda1(jj,4)) <= 1);
        end
        
        lambda0 = lambda0(lambda0(:,2) == 0,:);
        lambda0 = lambda0(:,1);
        lambda1;
        length(sortrows(unique(lambda1(:,4))))
        stop = (length(sortrows(unique(lambda1(:,4)))) == size(Z,2) + 1);                
    end
%     save('lambda1.mat','lambda1')
    array = array2table(lambda1);
    writetable(array,'LPDS table.xlsx')
    

    
    
    
    
    StV = QZ.*(X*ones(1,size(QZ,2)));
    UpsV = sqrt(sum(StV.^2)/(DT*N));
    % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            lambda = 1.1*2*sqrt(DT*N)*norminv(1-.05/(2*size(QZ,2)));
            PInitV = LassoShooting2(QZ, X , lambda, UpsV, 'Verbose', 0);
            IndInitV = abs(PInitV) > 0;
            ZV = QZ(:,IndInitV);
            RefResidV = X-QZ*PInitV;
            efsV = X-ZV*((ZV'*ZV)\(ZV'*X)) ;
            IndRefV = IndInitV;

            kk = 1;
            StRefV = QZ.*(RefResidV*ones(1,size(QZ,2)));
            UpsRefV = sqrt(sum(StRefV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            while norm(UpsRefV-UpsV) > 1e-4 && kk < maxIter,
                disp(kk);
                PRefV = LassoShooting2(QZ, X , lambda, UpsRefV , 'Verbose', 0);
                IndRefV = abs(PRefV) > 0;
                ZV = QZ(:,IndRefV);
                bfsV = ((ZV'*ZV)\(ZV'*X));
                efsV = X-ZV*bfsV;
                UpsV = UpsRefV;
                StRefV = QZ.*((X-QZ*PRefV)*ones(1,size(QZ,2)));
                UpsRefV = sqrt(sum(StRefV.^2)/(DT*N));    
                kk = kk+1;
            end
            sfsV = cluster_se(ZV,efsV,inv(ZV'*ZV),cluster_groups);

            % Structural equation
            StyV = QZ.*(Y*ones(1,size(QZ,2)));
            UpsyV = sqrt(sum(StyV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            PInityV = LassoShooting2(QZ, Y , lambda, UpsyV , 'Verbose', 0);
            IndInityV = abs(PInityV) > 0;
            ZyV = QZ(:,IndInityV);
            RefResidyV = Y-QZ*PInityV;
            efsyV = Y-ZyV*((ZyV'*ZyV)\(ZyV'*Y)) ;
            IndRefyV = IndInityV;

            kk = 1;
            StRefyV = QZ.*(RefResidyV*ones(1,size(QZ,2)));
            UpsRefyV = sqrt(sum(StRefyV.^2)/(DT*N));
            % lambda = 1.1*2*sqrt(2*DT*N*(log(2*(size(QZ,2)/.05))));
            while norm(UpsRefyV-UpsyV) > 1e-4 && kk < maxIter,
                disp(kk);
                PRefyV = LassoShooting2(QZ, Y , lambda, UpsRefyV, 'Verbose', 0);
                IndRefyV = abs(PRefyV) > 0;
                ZyV = QZ(:,IndRefyV);
                bfsyV = ((ZyV'*ZyV)\(ZyV'*Y));
                efsyV = Y-ZyV*bfsyV;
                UpsyV = UpsRefyV;
                StRefyV = QZ.*((Y-QZ*PRefyV)*ones(1,size(QZ,2)));
                UpsRefyV = sqrt(sum(StRefyV.^2)/(DT*N));    
                kk = kk+1;
            end
            sfsyV = cluster_se(ZyV,efsyV,inv(ZyV'*ZyV),cluster_groups);

            IndUnionV = max([IndRefyV IndRefV],[],2);
            IndUnionV(1:size(Q,2)) = 1;
            ZUnionV = QZ(:,IndUnionV);

            R2_1V = sum((ZUnionV*(ZUnionV\X)).^2)/(sum(X.^2));
            R2_2V = sum((ZUnionV*(ZUnionV\Y)).^2)/(sum(Y.^2)); 

            QZLASSO = [X ZUnionV];
            bLASSO = (QZLASSO'*QZLASSO)\(QZLASSO'*Y);
            eLASSO = Y - QZLASSO*bLASSO;
            ResLASSO = cluster_se(QZLASSO,eLASSO,inv(QZLASSO'*QZLASSO),cluster_groups);

           	beta0(jj) = bLASSO(1);
        	SE0(jj) = ResLASSO(1);
        	IndUnionV0(jj) = sum(IndUnionV)    
    
    