use "${folder}/Pre-regression data.dta", clear

	local Y "yearnavg7"
	local X "yearlpr"
	local Q "educ agew male college agewgt55 agewgt65 ones"
	local Z "interaction_group_dummies* X_*"
	local nx = 2
	local cl "obs"
	
	keep `Y' `X' `Q' `Z' `cl'
	rename `Y' Y
	rename `X' X
	local count = 1
	ds `Q'
	foreach qq in `r(varlist)' {
		rename `qq' Q`count'
		label var Q`count' "`qq'"
		local count = `count' + 1
	}
	local count = 1
	ds `Z'
	foreach zz in `r(varlist)' {
		rename `zz' Z`count'
		label var Z`count' "`zz'"
		local count = `count' + 1
	}
	rename `cl' clustervar	
	
	local Y "Y"
	local X "X"
	local Q "Q*"
	local Z "Z*"
	local nx = 2
	local cl "clustervar"		
	
	
	
	
	
	
	
	

	* Drop collinear Q variables
	reg `Y' `X' `Q', nocons
	matrix Betshort = e(b)'	
	local varnames: rownames Betshort
	foreach vv of local varnames {
		local vvo = subinstr("`vv'","o.","",.)
		if strpos("`vv'","o.") {
			drop `vvo'
		}
	}

	* Drop collinear Z variables
	
	ds `Q'
	local Qlist = `"`r(varlist)'"'
	foreach qq1 of local Qlist {
		local Qother = subinstr(`"`Qlist'"',`"`qq1'"',`""',.)
		reg `qq1' `Qother' `Z', nocons
		matrix Betshort = e(b)'	
		local varnames: rownames Betshort
		foreach vv of local varnames {
			local vvo = subinstr("`vv'","o.","",.)
			if strpos("`vv'","Z") & strpos("`vv'","o.") {
				di "`vvo'"
				drop `vvo'
			}
		}
	}
	
	* Drop collinear Z variables
	reg `Y' `X' `Q' `Z', nocons
	matrix Betshort = e(b)'	
	local varnames: rownames Betshort
	foreach vv of local varnames {
		local vvo = subinstr("`vv'","o.","",.)
		if strpos("`vv'","Z") & strpos("`vv'","o.") {
			di "`vvo'"
			drop `vvo'
		}
	}
	reg `Y' `X' `Q' `Z', nocons
	
	
	
	
	
	
	

	* Long regression
	reg `Y' `X' `Q' `Z', nocons
	gen fl_obsinlong = e(sample)
	local tstatL = _b[`X']/_se[`X']
	local coefL = _b[`X']		
	noi di "Coefficient: `coefL', T-statistic: `tstatL'"
	matrix Betlong = e(b)'
	local varnames: rownames Betlong
	assert (strpos("`varnames'","o.") == 0)

	* Short regression
	reg `Y' `X' `Q', nocons
	local tstatS = _b[`X']/_se[`X']
	local coefS = _b[`X']		
	noi di "Coefficient: `coefS', T-statistic: `tstatS'"	

* Variables for bivariate test
	keep if fl_obsinlong == 1
	keep `Y' `X' `Q' `Z' `cl'
	order Y X Q* Z* clustervar


* Statistics
	preserve
		* R-squared X on Y given Q
		reg Y Q*, nocons
		predict MqY, residuals
		reg X Q*, nocons
		predict MqX, residuals	
		reg MqY MqX, nocons
		local R2_XYgivenQ = `e(r2)'

		* R-squared Z on X given Q
		ds Z*
		foreach zz in `r(varlist)' {
			reg `zz' Q*, nocons
			predict Mq`zz', residuals
		}
		reg MqX MqZ*, nocons
		local R2_ZXgivenQ = `e(r2)'
	restore

* Data
	ds Q*
	local NQ: word count `r(varlist)'
	ds Z*
	local NZ: word count `r(varlist)'
	reg Y X Q* Z*, nocons cluster(clustervar)
	local N = `e(N)'
	matrix Betlong = e(b)'
	matrix Covlong = e(V)
	putexcel set "${folder}/Bivariatedata`nx'.xlsx", modify sheet("Covlong", replace)
	putexcel A1 = matrix(Covlong), names
	putexcel set "${folder}/Bivariatedata`nx'.xlsx", modify sheet("Betlong", replace)	
	putexcel A1 = matrix(Betlong), names	
	export excel using "${folder}/Bivariatedata`nx'.xlsx", firstrow(variables) sheet("Design") sheetreplace

	clear
	set obs 7
	gen Specification = ""
	gen Value = ""
	
	local count = 1
	replace Specification = "Paper" if _n == `count'
	replace Value = "Imbens, Rubin, Sacerdote (AER, September 2015)" if _n == `count'

	local count = `count' + 1
	replace Specification = "Baseline analysis" if _n == `count'
	replace Value = "Table 4, Column 2" if _n == `count'
	
	local count = `count' + 1
	replace Specification = "Y variable" if _n == `count'
	replace Value = "Post-lottery earnings (6yr average)" if _n == `count'

	local count = `count' + 1
	replace Specification = "X variable" if _n == `count'
	replace Value = "lottery prize" if _n == `count'		

	local count = `count' + 1
	replace Specification = "Q variable" if _n == `count'
	replace Value = "Years of educ., age at time of winning, dummies for {male, college, age at winning > 55, age at winning > 65}" if _n == `count'		

	local count = `count' + 1
	replace Specification = "Z variable" if _n == `count'
	replace Value = "Heterogeneous effect and dummies by >45, >55, >65 age bucket x college x gender" if _n == `count'		
	
	local count = `count' + 1	
	replace Specification = "Other" if _n == `count'
	replace Value = "CJN, baseline includes Y,X,Q, and Z is added for bivariate test" if _n == `count'	
	
	export excel using "${folder}/Bivariatedata`nx'.xlsx", firstrow(variables) sheet("Specification") sheetreplace

	clear
	set obs 11
	gen Statistic = ""
	gen Value = .

	local count = 1
	replace Statistic = "Number of observations" if _n == `count'
	replace Value = `N' if _n == `count'

	local count = `count' + 1	
	replace Statistic = "Number of baseline controls Q" if _n == `count'
	replace Value = `NQ' if _n == `count'

	local count = `count' + 1
	replace Statistic = "Number of potential controls Z" if _n == `count'
	replace Value = `NZ' if _n == `count'

	local count = `count' + 1					
	replace Statistic = "Betahat long" if _n == `count'
	replace Value = `coefL' if _n == `count'

	local count = `count' + 1	
	replace Statistic = "t-statistic long" if _n == `count'
	replace Value = . if _n == `count'

	local count = `count' + 1	
	replace Statistic = "Betahat short" if _n == `count'
	replace Value = `coefS' if _n == `count'

	local count = `count' + 1	
	replace Statistic = "t-statistic short (Matlab)" if _n == `count'
	replace Value = . if _n == `count'

	local count = `count' + 1	
	replace Statistic = "t-statistic short (Stata)" if _n == `count'
	replace Value = `tstatS' if _n == `count'

	local count = `count' + 1		
	replace Statistic = "R-squared of X on Y given Q" if _n == `count'
	replace Value = `R2_XYgivenQ' if _n == `count'

	local count = `count' + 1	
	replace Statistic = "R-squared of Z on X given Q" if _n == `count'
	replace Value = `R2_ZXgivenQ' if _n == `count'

	local count = `count' + 1	
	replace Statistic = "Correlation between long and short" if _n == `count'
	replace Value = . if _n == `count'	

	export excel using "${folder}/Bivariatedata`nx'.xlsx", firstrow(variables) sheet("Statistic") sheetreplace
	keep if strpos(Statistic,"Betahat long") | strpos(Statistic,"Betahat short") | strpos(Statistic,"t-statistic short (Stata)")
	export excel using "${folder}/Bivariatedata`nx'.xlsx", firstrow(variables) sheet("QC") sheetreplace

*** EOF ***
