module MCmod
	use cvmod
	use globals
	implicit none
	
	integer,parameter	:: ntests=7
	
	integer, parameter	:: n=500,k=200
	real	:: eta=0.1,rho2=0.95,beta=0.00,kappa=10,nu=1
	real, parameter	:: kbarlist(2)=[0.20,0.5]*sqrt(real(n))
	contains
	
	subroutine drawMC(y,x,z)
		use rlse_int
		real	:: y(n),x(n),Z(n,k)
		integer	:: i
		real	:: ZZi(k,k),e(n),s(n),s0(n),a,b,c,Ze(k),Zs(k)
		
		
		call rnnoa(y)
		call rnnoa(x)
		do i=1,k
			call rnnoa(z(:,i))
		enddo
		s0=sum(z(:,1:nint(k*eta)),dim=2)
		s=s0-sum(s0)/n
		y=y+s0*kappa/sqrt(sum(s**2))
		
		ZZi=invertpd(matmul(transpose(Z),Z))
		e=x-sum(x)/n
		s0=sum(z(:,nint(k*eta*(1-nu)+1):nint(k*eta*(2-nu))),dim=2)
		s=s0-sum(s0)/n
		Ze=matmul(transpose(Z),e)
		Zs=matmul(transpose(Z),s)
		c=sum(Ze*matmul(ZZi,Ze))-rho2*sum(e**2)
		b=2*(sum(Ze*matmul(ZZi,Zs))-rho2*sum(e*s))
		a=sum(Zs*matmul(ZZi,Zs))-rho2*sum(s*s)
		x=x+s0*(-b+sqrt(b**2-4*a*c))/(2*a)
		y=y+beta*x
	end subroutine
	
	function gettests(y,x,z) result(val)
		use rlse_int
		real	:: y(n),x(n),Z(n,k)
		real	:: val(ntests)
		real	:: beta_s(2), beta_l(k+2) ,s2,e(n),xm(n),ga(k+1),x2(2,n),Om(2,2),nxm
		real	:: Kdel,kap,dkap
		integer	:: j

		call rlse(y,reshape(x,[n,1]),beta_s)
		e=y-x*beta_s(2)-beta_s(1)
		xm=x-sum(x)/n
		nxm=sqrt(sum(xm**2))
		x2(2,:)=xm
		call rlse(y,reshape([x,z],[n,k+1]),beta_l)
		e=y-x*beta_l(2)-beta_l(1)-matmul(z,beta_l(3:))
		call rlse(x,z,ga)
		xm=x-ga(1)-matmul(z,ga(2:))
		x2(1,:)=xm
		Om=getCJN(e,x2,reshape([z,[(1.0,j=1,n)]],[n,k+1]))
		val(1)=boole(abs(beta_s(2))/sqrt(Om(2,2))  >1.96)
		val(2)=1.96*sqrt(Om(2,2))
		val(3)=boole(abs(beta_l(2))/sqrt(Om(1,1))  >1.96)
		val(4)=1.96*sqrt(Om(1,1))

		Kdel=sqrt(rho2)*kappa/nxm
		val(5)=getLR(Kdel,beta_l(2),beta_s(2),Om)
		val(6)=getLRlength(Kdel,beta_l(2),beta_s(2),Om)
		
		if(val(1)==0) then
			val(7)=0
		else if(val(1)==1 .and. val(2)==1) then
			val(7)=1.0/0.0
		else 
			kap=40
			dkap=20
			do j=1,20
				Kdel=sqrt(rho2)*kap/nxm
				if(getLR(Kdel,beta_l(2),beta_s(2),Om)==1.0) then
					kap=kap+dkap
				else
					kap=kap-dkap
				endif
				dkap=0.5*dkap
			enddo
			val(7)=kap
		endif
		val([1,3,5])=1.0-val([1,3,5])
		val([2,4,6])=2*val([2,4,6])
		val(7)=val(7)/sqrt(real(n))
	end function
	
	function getLRlength(Kdel,bl,bs,Om) result(val)
		real	:: Kdel,bl,bs,Om(2,2),val
		real	:: b,delb,b0
		integer	:: j
		b0=getbhat(Kdel,bl,bs,Om)
		if(getLR(Kdel,bl-b0,bs-b0,Om)==1.0) then
			print *,"rejecting at MLE"
			stop
		endif
		delb=5*sqrt(Om(1,1))
		b=b0+delb
		do j=1,20
			delb=delb/2
			if(getLR(Kdel,bl-b,bs-b,Om)==1.0) then
				b=b-delb
			else
				b=b+delb
			endif	
		enddo
		val=b
		delb=5*sqrt(Om(1,1))
		b=b0-delb
		do j=1,20
			delb=delb/2
			if(getLR(Kdel,bl-b,bs-b,Om)==1.0) then
				b=b+delb
			else
				b=b-delb
			endif	
		enddo
		val=0.5*(val-b)
	end function

	function getCJN(e,x2,z) result(val)
		use rlse_int
		real	:: e(n),x2(2,n),Z(n,k)
		real	:: val(2,2)
		real	:: m(n,n),xx2(2,2)
		integer	:: i,j

		m=-matmul(Z,matmul(invertpd(matmul(transpose(Z),Z)),transpose(Z)))
		do i=1,n
			m(i,i)=m(i,i)+1
		enddo
		m=invertgen(m*m)
		val=0
		do i=1,n
			do j=1,n
				val=val+m(j,i)*e(j)**2*reshape([x2(1,i)**2,x2(1,i)*x2(2,i),x2(1,i)*x2(2,i),x2(2,i)**2],[2,2])
			enddo
		enddo
		xx2=reshape([1/sum(x2(1,:)**2),0.0,0.0,1/sum(x2(2,:)**2)],[2,2])
		val=matmul(xx2,matmul(val,xx2))
	end function
	
	function getbhat(Kdel,bl,bs,Om) result(val)
		real	:: kdel,bl,bs,Om(2,2),val
		real	:: omsd,wb,kg,y1,y2,Omr,ysign
		
		Omr=Om(1,2)/sqrt(Om(1,1)*Om(2,2))
		omsd=sqrt(om(1,1)*om(2,2)-om(1,2)**2)
		kg=sqrt(om(1,1))/omsd
		wb=(om(1,1)-om(1,2))/omsd
		y1=bl/sqrt(Om(1,1))
		y2=(wb/sqrt(Om(1,1))-kg)*bl+kg*bs
		ysign=1
		if(wb<0) then
			y1=-y1
			wb=-wb
			ysign=-1
		endif
		kg=kg*Kdel
		if(kg+wb*y1<y2) then
			val=(-(kg*wb) + y1 + wb*y2)/(1 + wb**2)
		else if(kg + y2.le.wb*y1) then
			val=(y1 + wb*(kg + y2))/(1 + wb**2)
		else
			val=y1
		endif
		val=ysign*val*sqrt(Om(1,1))
	end function
	
	
	function getLR(Kdel,bl,bs,Om) result(val)
		real	:: kdel,bl,bs,Om(2,2),val
		real	:: omsd,wb,kg,y1,y2,Omr
		
		Omr=Om(1,2)/sqrt(Om(1,1)*Om(2,2))
		omsd=sqrt(om(1,1)*om(2,2)-om(1,2)**2)
		kg=sqrt(om(1,1))/omsd
		wb=(om(1,1)-om(1,2))/omsd
		y1=bl/sqrt(Om(1,1))
		y2=(wb/sqrt(Om(1,1))-kg)*bl+kg*bs
		if(wb<0) then
			y1=-y1
			wb=-wb
		endif
		kg=kg*Kdel
		val=getrLR(y1,y2,wb,kg)>getcv(kg,wb)
	end function
	
	function getrLR(y1,y2,wb,kg) result(val)
		real	:: y1,y2,wb,kg,val
		real	:: m0,m1
		if ((y2>0 .and. kg>y2) .or. (y2<0 .and. kg+y2>0)) then
			m0=y1**2
		else if(kg<y2) then
			m0=y1**2+(kg-y2)**2
		else
			m0=y1**2+(kg+y2)**2
		endif
		if	(kg+wb*y1<y2) then
			m1=(kg+wb*y1-y2)**2/(1+wb**2)
		else if ((kg+y2<wb*y1 .and. ((y1>0 .and. y2<0) .or. y2>0)) .or. (y1<0 .and. kg+y2<wb*y1)) then 
			m1=(kg-wb*y1+y2)**2/(1+wb**2)
		else	
			m1=0
		endif		
		val=m0-m1
	end function		
		
	subroutine MCall
		real, allocatable	:: ftab(:,:),fxtab(:,:)
		integer	:: ieta,inu,irho,ibeta,ikappa
		integer, parameter	:: nsim=20000
		real	:: outtab(ntests,nsim)
		real	:: y(n),x(n),Z(n,k)
		integer	:: i

		allocate(ftab(0,0),fxtab(0,0))
		do ikappa=1,2
		kappa=kbarlist(ikappa)
		do ieta=0,1
		eta=merge(0.1,0.3,ieta==0)
		do irho=0,1
		rho2=merge(0.6,0.95,irho==0)
		do inu=0,2
		nu=inu*0.5
!$omp parallel do private(y,x,Z) 
			do i=1,nsim	
				call drawMC(y,x,z)
				outtab(:,i)=gettests(y,x,z)
			enddo
			ftab=reshape([ftab,kappa,eta,rho2,nu,sum(outtab([1,2,3,4],:),dim=2)/nsim,-1.0,-1.0,sum(outtab([5,6],:),dim=2)/nsim,quantile_v(outtab(7,:),[0.25,0.5,0.75])],[15,size(ftab,2)+1])
			fxtab=reshape([fxtab,quantile_v(outtab(2,:),[0.25,0.5,0.75]),quantile_v(outtab(4,:),[0.25,0.5,0.75]),quantile_v(outtab(6,:),[0.25,0.5,0.75])],[9,size(fxtab,2)+1])
			call mdisp(transpose(ftab))
			call mdisp(transpose(fxtab))
		enddo
		enddo
		enddo
		enddo
		
	end subroutine
	
end module
	
	
program mfort
	use Mcmod
	use omp_lib	
	implicit none
	integer	:: i
	
!$omp parallel do
	do i=1,50
		call rnset(20+134753*omp_get_thread_num())
	enddo
	call rnset(18000)
	
	call savecvs
	call inittime
	print *,getcv(1.0,1.0)
	
	call MCall

end program	
	