module globals
	use myfuncs
	use dotops
	
	character(len=100)	:: dir="c:/dropbox/mystuff/sparsity/replication/"
	
end module

module cvmod
	use globals
	implicit none
	
	private
	public	:: getcv , getcvs, savecvs
	
    integer, parameter  :: nlevel=3
    real, parameter 	:: levellist(nlevel)=[0.01,0.05,.1]
	integer, parameter  :: n1=40,n2=40
    
	integer, parameter	:: nsim=1000000

	real, allocatable	:: Ys(:,:)

	contains
	
    subroutine prep
        integer     :: l

        allocate(Ys(2,nsim))
        do l=1,nsim
            call rnnoa(Ys(:,l))
        enddo
        print *,"done with prep"
    end subroutine
    
    function getLR(kg,wb,Y) result(val)
        real    :: kg,wb,Y(2),val
        
        real    :: y1,z2,m0,m1
        y1=Y(1);z2=Y(2)
        
        if ( (z2<0 .and. kg+z2>0) .or. (kg+z2<0 .and. 2*kg+z2>0) ) then
            m0=y1**2
        else
            if(z2>0) then
                m0=y1**2+z2**2
            else
                m0=y1**2+(2*kg+z2)**2
            endif
        endif
        if (wb*y1<z2) then
            m1=(z2-wb*y1)**2/(1+wb**2)
        else
            if ( (2*kg+z2<wb*y1 .and.  ((y1>0 .and. kg+z2<0) .or. kg+z2>0)) .or. (y1<0 .and. 2*kg+z2<wb*y1) ) then
                m1=(2*kg-wb*y1+z2)**2/(1+wb**2)
            else
                m1=0
            endif
        endif
        val=m0-m1
    end function            
        
    function getcvs(kg,wb) result(val)
        real    :: kg,wb,val(nlevel)
        real, allocatable   :: stats(:)
        integer :: l

        allocate(stats(nsim))
!$omp parallel do        
        do l=1,nsim
            stats(l)=getLR(kg,wb,Ys(:,l))
        enddo
        val=quantile(stats,1-levellist)
	end function
	
	subroutine savecvs
		real	:: outtab(nlevel,n1,n2)
		real    :: m
		integer :: i1,i2,i
		real    :: x1,x2

		call prep
    
		do i1=1,n1
			do i2=1,n2
				x1=real(i1)/n1; x2=real(i2)/n2
				m=200.0**(-1+2*x1)
				outtab(:,i1,i2)=getcvs(m*sin(.5*pi*x2),m*cos(.5*pi*x2))
			enddo
		enddo
		do i=1,nlevel
			call savemat(trim(dir)//"cvmat_"//convtos(nint(100*levellist(i)))//".txt",outtab(i,:,:))
		enddo
	end subroutine
    
	function getcv(kg,wb) result(val)
		real	:: kg,wb,val
		logical	:: init=.true.
		real,save    :: outtab(n1,n2)
		real    :: m
		integer :: i1,i2
		real    :: x1,x2,w1,w2

		if(init) then
			outtab=loadmat(trim(dir)//"cvmat_5.txt")
			init=.false.
			val=-1
			return
		endif
		x1=0.5*(0.5*log(kg**2+wb**2)/log(200.0)+1)
		x2=2*atan(kg/wb)/pi
		i1=x1*n1;w1=x1*n1-i1;i1=max(min(i1,n1-1),1) 
		i2=x2*n2;w2=x2*n2-i2;i2=max(min(i2,n2-1),1)
		val=sum([outtab(i1:i1+1,i2:i2+1)]*[(1-w1)*(1-w2),w1*(1-w2),w2*(1-w1),w1*w2])
	end function
	
end module
	
