clear all
clc
%cd('<Working directory>');
disp(datestr(now))
addpath(genpath(pwd))

%% SECTION 1: Parameters that do not depend on design

% Set N,K,alpha
par.N = 500;
par.K = 200;

% Set error variance
std_e = linspace(1,1,par.N)';
var_e = std_e.^2;
par.var_e = var_e;

% Set alternative, null values of beta
beta_alt = .06;
beta_null = 0;

% Set level, load Critical Values
par.alpha = .05;
% load('CV_vals.mat')
%i_CV = 1*(par.alpha==.01)+2*(par.alpha==.05)+3*(par.alpha==.1);
%CV_struct = CV(i_CV);

%% SECTION 2: Parameters that vary with design

% # draws of design per parameter
n_MCcomp =  20000;
MCstructure = struct;
i = 1;
kappavalues = [.20,.5]*sqrt(par.N) %[5,10,20]; %[temp,.32]; %set imposed R2 values
for kappa = kappavalues 
for eta = [.1,.30]    
for rhosq = [.6,.95]
for nu = [0,.5,1]
    MCstructure(i).kappa = kappa;
    MCstructure(i).rhosq = rhosq;
    MCstructure(i).nu = nu;
    MCstructure(i).eta = eta;
    i = i+1;

end
end
end
end

disp(['total number of designs ',num2str(length(MCstructure))])

%% SECTION 3: Generate size, power for LPDS
for ispecs = 1:length(MCstructure)

%     ispecs = 1;
    kappa = MCstructure(ispecs).kappa;
    rhosq = MCstructure(ispecs).rhosq;
    nu = MCstructure(ispecs).nu;
    eta = MCstructure(ispecs).eta;

    % matrices for storing size, power conditional on design
    LPDS_Beta_Null = zeros(n_MCcomp,1);
    LPDS_Beta_H1 = zeros(n_MCcomp,1);
    LPDS_SE_Null = zeros(n_MCcomp,1);    
    LPDS_SE_H1 = zeros(n_MCcomp,1);        
    LPDS_Size = zeros(n_MCcomp,1);
    LPDS_Power = zeros(n_MCcomp,1);

    % loop over different draws of design
    tempY0 = zeros(n_MCcomp,1);
    tempY1 = zeros(n_MCcomp,1);

    parfor i_MCcomp = 1:n_MCcomp

        % Generate (Y,X,Z)
        [Y1,Y0,X,Z,gamma] = Gen_Obs_sparse_UC( beta_alt, beta_null, kappa, rhosq, nu, eta, var_e, par);

        %LPDS
        [LPDS_Size(i_MCcomp),LPDS_Beta_Null(i_MCcomp),LPDS_SE_Null(i_MCcomp)] = LPDS_test(Y0(:,1),X,Z,par);
%        [LPDS_Power(i_MCcomp),LPDS_Beta_H1(i_MCcomp),LPDS_SE_H1(i_MCcomp)] = LPDS_test(Y1(:,1),X,Z,par);
    end
    
    MCstructure(ispecs).LPDS_Cov = 1-mean(LPDS_Size);

 %   MCstructure(ispecs).LPDS_power = mean(LPDS_Power);
    
%    MCstructure(ispecs).LPDS_Beta_Null = LPDS_Beta_Null;
  %  MCstructure(ispecs).LPDS_Beta_H1 = LPDS_Beta_H1;
    MCstructure(ispecs).LPDS_Length  = 2*1.96*mean(LPDS_SE_Null);    
    MCstructure(ispecs).LPDS_LengthQ1  = 2*1.96*quantile(LPDS_SE_Null,.25);    
    MCstructure(ispecs).LPDS_LengthQ2  = 2*1.96*quantile(LPDS_SE_Null,.50);    
    MCstructure(ispecs).LPDS_LengthQ3  = 2*1.96*quantile(LPDS_SE_Null,.75);    
  %  MCstructure(ispecs).LPDS_SE_H1 = LPDS_SE_H1;        

    disp(['done with ',num2str(ispecs)])
    disp(datestr(now))

end

save('MCLPDS.mat','MCstructure');
% 
% for ispecs = 1:length(MCstructure)
%     %MCstructure(ispecs).LPDS_Cov = 1- mean(MCstructure(ispecs).LPDS_Beta_Null);
%    % MCstructure(ispecs).LPDS_Beta_H1_avg = mean(MCstructure(ispecs).LPDS_Beta_H1);    
%    % MCstructure(ispecs).LPDS_Length = 2*1.96*mean(MCstructure(ispecs).LPDS_SE_Null);
%    % MCstructure(ispecs).LPDS_SE_H1_avg = mean(MCstructure(ispecs).LPDS_SE_H1); 
% end
% 
% LPDS_Simulations = MCstructure;
% LPDS_Simulations = rmfield(LPDS_Simulations, 'LPDS_Beta_Null');
% %LPDS_Simulations = rmfield(LPDS_Simulations, 'LPDS_Beta_H1');
% LPDS_Simulations = rmfield(LPDS_Simulations, 'LPDS_SE_Null');
% %LPDS_Simulations = rmfield(LPDS_Simulations, 'LPDS_SE_H1');
% writetable(struct2table(LPDS_Simulations),'LPDSMC_CI.xlsx')
% %save('MCLPDSavg.mat','MCstructure')
