module globals
	use myfuncs
	use dotops
	
	character(len=100)	:: dir="c:/dropbox/mystuff/sparsity/replication/"

	integer, parameter  :: nr=3,nkt=5
	real                :: ktab(nkt)=[0.001,1.0,3.0,10.0,30.0]
    real                :: rlist(nr)=[0.5,0.9,0.99]
	
	real				:: kappa=10, rho=0.95
	integer, parameter	:: nsim=500000, nY=2, nth=2
	real				:: kappaimp
	
	real, parameter		:: level=0.05
	

	integer, parameter	:: n0=101
	integer				:: nimp
	real, allocatable	:: Ys(:,:),densimp(:), dens0(:,:), densalt(:), dens1(:,:)
	logical, allocatable	:: LRtest(:),opttest(:)	
	
	real				:: lam(n0), d0list(n0)
	real				:: Sigi(2,2), Sigiimp(2,2)
	real				:: sbimp=-100
	real				:: cvLR

	real, allocatable	:: mmtesttab(:,:)
	
	logical				:: useAKtest=.false.

end module

module compute
	use globals
	implicit none
	
	contains
	
	function getdensimp(m,Y) result(val)
		real	:: Y(nY), val
        real    :: m
		real	:: a11,a12
        real    :: S(2,2)
        
		S=Sigiimp
        a11=S(2,2)
        a12=sum(S(2,:)*y)
        val=-.5*sum(Y*matmul(S,Y))-.5*log(1+sbimp)-log(2*m)+(a12**2/a11 - Log(2*a11) + Log(Pi) + 2*Log(-Erf((a12 - a11*m)/(Sqrt(2.0)*Sqrt(a11))) + Erf((a12 + a11*m)/(Sqrt(2.0)*Sqrt(a11)))))/2.
    end function

	subroutine prep
		integer	:: l,i0,j,i
		real	:: Y(nY),h,u(1)
		real		:: b,d,Sigchol(2,2)
		
		Sigi=invertpd(reshape([1/(1-rho**2),1.0,1.0,1.0],[2,2]))
		Sigiimp=invertpd(reshape([1/(1-rho**2)+sbimp,1.0+sbimp,1.0+sbimp,1.0+sbimp],[2,2]))
		Sigchol=choleski(reshape([1/(1-rho**2)+sbimp,1.0+sbimp,1.0+sbimp,1.0+sbimp],[2,2]))
		do l=1,nsim
			call rnnoa(Y)
			Y=matmul(Sigchol,Y)
			call rnun(U)
			Y(2)=Y(2)+kappaimp*(2*U(1)-1)
			densimp(l)=getdensimp(kappaimp,Y)
			Ys(:,l)=Y
		enddo			
    end subroutine

    subroutine setdens0(kappa)
        real    :: kappa
        integer :: l,j
        real    :: Y(nY)
        
		d0list=kappa*[((j-1.0)/(n0-1),j=1,n0)]
!$omp parallel do private(Y,j) 	
		do l=1,nsim
			Y=Ys(:,l)
			do j=1,n0
				dens0(j,l)=exp(getdens0(d0list(j),Y)-densimp(l))
			enddo
        enddo
!		print *,"IS weights test"
!		call mdisp(sum(dens0,dim=2)/nsim)
    end subroutine
	
	function getdens0(d,Y) result(val)
		real	:: d, Y(nY), val, v(2)
		integer	:: i
		
		val=-1000
		do i=-1,1,2
			v=[0.0,i*d]
			val=logsumexp([val,-.5*sum((Y-v)*matmul(Sigi,Y-v))])
		enddo
		val=val+log(0.50)
	end function
	
	subroutine setlam()
		real	:: RP(n0), lamstep
		real	:: oldRP(n0), lamfac(n0), cv
		integer	:: l,j

		lamfac=4
		lam=1
		oldRP=level
		do j=1,500
			RP=0.0
!$omp parallel do reduction(+:RP)
			do l=1,nsim
				if(dot_product(dens0(:,l),lam)<densalt(l)) RP=RP+dens0(:,l)
			enddo
			RP=RP/nsim
			lam=lam*exp(lamfac*(RP-level))
			where((RP-level)*(oldRP-level)>0)
				lamfac=lamfac*1.03
			elsewhere
				lamfac=lamfac/2.0
			endwhere
			oldRP=RP
			if(j>400) lamfac=0.1
			call setbounds(lamfac,0.01,20.0)
		enddo		
		opttest=matmul(lam,dens0)<densalt
	end subroutine
	
	function getdensalt(d,Y) result(val)
		real	:: d,Y(2),val
		real	:: r12
		r12=1/(1-rho**2)
		val=sqrt(2*Pi)*exp((-(d + y(1) - y(2))**2/(2.*(-1 + r12))))
	end function
	
	function getavdalt(m,Y) result(val)
		real	:: m,Y(2),val
		val=(0.5/m)*sqrt(2*Pi)*(Sqrt(Pi)*rho*(Erf((Sqrt(1 - rho**2)*(m + y(1) - y(2)))/(Sqrt(2.0)*rho)) + Erf((Sqrt(1 - rho**2)*(m - y(1) + y(2)))/(Sqrt(2.0)*rho))))/Sqrt(2 - 2*rho**2)
	end function
	
    subroutine setdensalt
		integer	:: l
!$omp parallel do
		do l=1,nsim
			densalt(l)=getavdalt(kappa,Ys(:,l))/exp(densimp(l))
		enddo
	end subroutine
	
	subroutine setLRtest
		real	:: stats(nsim)
		integer	:: l,j
		real	:: cvs(n0)
		if(useAKtest) then
!$omp parallel do
			do l=1,nsim
				LRtest(l)=getAKtest(Ys(:,l))
			enddo
			return
		endif

!$omp parallel do
		do l=1,nsim
			stats(l)=getLRstat(Ys(:,l))
		enddo
!$omp parallel do 
		do j=1,n0
			cvs(j)=getcv(stats,dens0(j,:),level)
		enddo
		cvLR=maxval(cvs)
		LRtest=stats>cvLR
	end subroutine
		
	
	function getcv(stats,w,alpha) result(val)
        use SVRGP_INT
        real    :: stats(nsim), w(nsim),alpha, val
        real    :: sort(nsim),p
        integer :: inds(nsim),l
        
        inds=[(l,l=1,nsim)]
        call svrgp(stats,sort,inds)
        p=w(inds(nsim))
        do l=nsim-1,1,-1
            if(p+w(inds(l))>alpha*nsim) then
                val=sort(l+1)
                return
            endif
            p=p+w(inds(l))
        enddo
        print *,"error in getcv: weights sum up to less than alpha"
        stop
	end function

	function getLRstat(Y) result(val)
		real	:: Y(nY),val
		real	:: m0,m1,t1,t2,y1,y2
		y1=Y(1);y2=Y(2)
		if(kappa+y1<rho**2*y1+y2) then
			m0=(kappa**2-(-1+rho**2)*y1**2+2*(-1+rho**2)*y1*y2+y2**2-2*kappa*((-1+rho**2)*y1+y2))/rho**2
		else
			if(kappa+rho**2*y1+y2<y1) then
				m0=(kappa**2-(-1+rho**2)*y1**2+2*(-1+rho**2)*y1*y2+y2**2+2*kappa*((-1+rho**2)*y1+y2))/rho**2
			else
				m0=(1-rho**2)*y1**2
			endif
		endif
		if(kappa+y2<y1) then
			m1=(1-rho**2)*(kappa-y1+y2)**2/rho**2
		else
			if(kappa+y1<y2) then
				m1=(1-rho**2)*(kappa+y1-y2)**2/rho**2
			else
				m1=0
			endif
		endif
		val=m0-m1
	end function
	
	function getlength(tst,dalt) result(val)
		logical	:: tst(nsim)
		real	:: dalt(nsim),val,s(2)
		integer	:: l

		s=0
		do l=1,nsim
			if(.not. tst(l)) s=s+[dalt(l),dalt(l)**2]
		enddo
		s=s/nsim
		val=s(1)
	end function
	
    subroutine comp
	    integer	:: i, ir, j
        real                :: outtab(nr,nkt,4)

        outtab=-1
        do ir=1,nr
            rho=rlist(ir)
	        do i=1,nkt
		        kappa=ktab(i)*rho
				call rnset(14)	
				kappaimp=kappa+0/(1-rho**2)
				sbimp=5.0/(1-rho**2)
				call prep()
				call setdens0(kappa)
	
				call setdensalt
				call setlam
				call setLRtest
				outtab(ir,i,:)=lendiff()
            enddo
        enddo
		print *,"Weighted Average Beta Length"
		call mdisp(outtab(:,:,1).clr.outtab(:,:,2))
		print *,"Min and Max of Beta Length ratio relative to Long Regression"
		call mdisp(outtab(:,:,3).clr.outtab(:,:,4))
    end subroutine
	
	function lendiff result(tab)
        real            :: tab(4)
		integer, parameter	:: ndgrid=21
		real			:: dgrid(ndgrid)
		real    		:: ptab(size(dgrid))
        logical,allocatable		:: regtest(:)
		real, allocatable	:: densd(:)
        integer         :: i,l,t,j,tdiff
        real            :: wpower(2)
        
		allocate(densd(nsim))
		dgrid=[((i-1)*(kappa)/(ndgrid-1),i=1,ndgrid)]
		do i=1,ndgrid
!$omp parallel do
			do l=1,nsim
				densd(l)=getdensalt(dgrid(i),Ys(:,l))/exp(densimp(l))
			enddo
			ptab(i)=getlength(LRtest,densd)/(2*1.96/sqrt(1-rho**2))
		enddo
        tab=[getlength(LRtest,densalt),getlength(opttest,densalt),minval(ptab),maxval(ptab)]
    end function
	
	function getAKtest(Y) result(val)
		real	:: Y(2)
		logical	:: val
		integer	:: i
		real	:: v(2)=[1,1000]
		if(minval(abs(matmul(mmtesttab(:,1:2),v)-sum([rho,kappa/rho]*v)))>1E-6) then
			print *,rho, kappa/rho, "unsupported kappa/rho combo"
			stop
		endif
		i=minloc(abs(matmul(mmtesttab(:,1:2),v)-sum([rho,kappa/rho]*v)),dim=1)
		val=abs(sum(mmtesttab(i,3:4)*Y))>mmtesttab(i,5)
	end function	
	
	function getAKest(Y) result(val)
		real	:: Y(2),val
		integer	:: i
		real	:: v(2)=[1,1000]
		if(minval(abs(matmul(mmtesttab(:,1:2),v)-sum([rho,kappa/rho]*v)))>1E-6) then
			print *,rho, kappa/rho, "unsupported kappa/rho combo"
			stop
		endif
		i=minloc(abs(matmul(mmtesttab(:,1:2),v)-sum([rho,kappa/rho]*v)),dim=1)
		val=sum(mmtesttab(i,3:4)*Y)
	end function	

	subroutine setminmaxtest
		integer	:: l,i,j
		real	:: RP(n0),lgth(n0),lam1(n0)
		
!$omp parallel do private(i)
		do l=1,nsim
			do i=1,n0
				dens1(i,l)=(.5*getdensalt(d0list(i),Ys(:,l))+.5*getdensalt(-d0list(i),Ys(:,l)))/exp(densimp(l))
			enddo
		enddo
		lam=1
		lam1=1.0/n0

		do j=1,1500
			RP=0.0;lgth=0
!$omp parallel do reduction(+:RP,lgth)
			do l=1,nsim
				if(dot_product(dens0(:,l),lam)<dot_product(dens1(:,l),lam1)) then
					RP=RP+dens0(:,l)
				else
					lgth=lgth+dens1(:,l)
				endif
			enddo
			RP=RP/nsim
			where(lgth==0) lgth=minval(lgth)
			if(all(lgth==0)) lgth=1
			lgth=lgth/nsim
			lam=lam*exp(4*(RP-level))
			lam1=lam1*(n0*lgth/sum(lgth))**2
			lam1=n0*lam1/sum(lam1)
			call setbounds(lam1,0.0001,1000.0)
		enddo		
		opttest=matmul(lam,dens0)<matmul(lam1,dens1)

	end subroutine
	
	subroutine comp_minmax
	    integer		:: i, ir, j
        real        :: ml(2),outtab(nr,nkt,2)
        outtab=-1
        do ir=1,nr
            rho=rlist(ir)
	        do i=1,nkt
		        kappa=ktab(i)*rho
				call rnset(14)	
				kappaimp=kappa+2
				kappaimp=kappa+0
				sbimp=5.0/(1-rho**2)
				
				call prep()
				call setdens0(kappa)
				call setLRtest
				call setminmaxtest
				ml=0
				do j=1,n0
					ml(1)=max(ml(1),getlength(LRtest,dens1(j,:)))
					ml(2)=max(ml(2),getlength(opttest,dens1(j,:)))
				enddo
				outtab(ir,i,:)=ml
            enddo
        enddo
		print *,"Max beta Length"
        call mdisp(outtab(:,:,1).clr.outtab(:,:,2))
	end subroutine
	
	subroutine MSEcomp
	    integer	:: i, ir, j, l
        real                :: outtab(nr,nkt,2)
		real				:: bhats(2), mse(2), y(2)

        outtab=-1
        do ir=1,nr
            rho=rlist(ir)
	        do i=1,nkt
		        kappa=ktab(i)*rho
				call rnset(14)	
				kappaimp=kappa
				sbimp=0
				call prep()
				call setdens0(kappa)
				call setLRtest()
				mse=0
!$omp parallel do reduction(+:mse) private(y,bhats)
				do l=1,nsim
					y=ys(:,l)
					bhats=get_ests(y)
					mse=mse+bhats**2 
				enddo
				mse=mse/nsim
				outtab(ir,i,:)=mse
            enddo
        enddo
		print *,"weighted avearage MSE"
		call mdisp(outtab(:,:,1).clr.outtab(:,:,2))
    end subroutine

	function get_ests(y) result(val)
		real	:: y(2), val(2)
		real	:: y1,y2
		real	:: r12,kar,exb
		
		y1=y(1); y2=y(2)
		if(useAKtest) then
			val(1)=getAKest(Y)
		else
			val(1)=getLRest(y)
		endif
		r12=1/(1-rho**2)
		kar=kappa
		exb=-((kar**2 + (y1 - y2)**2)/(-1 + r12))
		val(2)=exp(((kar - y1 + y2)**2/(2.*(-1 + r12)))+exb)-exp((kar + y1 - y2)**2/(2.*(-1 + r12))+exb)
		val(2)=val(2)*Sqrt(2/Pi)*Sqrt(-1 + r12)
		val(2)=val(2)/(Erf((kar + y1 - y2)/(Sqrt(2.0)*Sqrt(-1 + r12))) + Erf((kar - y1 + y2)/(Sqrt(2.0)*Sqrt(-1 + r12))))
		val(2)=val(2)+y1
	end function
	
	function getLRest(y) result(val)
		real	:: y(2), val
		real	:: y1,y2,mle,b,delb
		integer	:: j
		y1=y(1); y2=y(2)
		if((kappa + y1.ge.y2.and.(kappa + y2.eq.y1.or.y1.le.y2)).or.(y1.ge.y2.and.kappa + y2.ge.y1)) then
			mle=y1
		else if (kappa + y1.le.y2) then
			mle=-(kappa) + y2
		else
			mle=kappa + y2
		endif
		delb=sqrt(5.0/(1-rho**2))
		b=mle+delb
		do j=1,20
			delb=.5*delb
			if(getLRstat(y-b)>cvLR) then
				b=b-delb
			else
				b=b+delb
			endif
		enddo
		val=b
		delb=sqrt(5.0/(1-rho**2))
		b=mle-delb
		do j=1,20
			delb=.5*delb
			if(getLRstat(y-b)>cvLR) then
				b=b+delb
			else
				b=b-delb
			endif
		enddo
		val=.5*(val+b)
	end function
		
	subroutine lengthkappa
	    integer	:: i, ir, j, ikmax,l,is,in0,dn0
        integer, parameter  :: nkt=21,nkmax=4
	    real                :: kmaxtab(nkmax)=[1.0,3.0,10.0,30.0]
	    real                :: ktab(nkt)=[((i-1.0)/(nkt-1.0),i=1,nkt)]
        real                :: outtab(2,nr,nkmax),tab(4)
        real                :: kmax
		real				:: pwer,pwer2,y(2)

		do is=0,0
			outtab=0
			do ir=1,nr
				rho=rlist(ir)
				do ikmax=1,nkmax
					kmax=kmaxtab(ikmax)*rho
					call rnset(14)	
					kappaimp=kmax
					sbimp=is*5
					call prep()
!$omp parallel do private(y)
					do l=1,nsim
						y=Ys(:,l)
						densalt(l)=exp(getdens0(kmax,y)-densimp(l))
					enddo
!					print *,"densalt IS check",sum(densalt)/nsim
					call setdens0(kappaimp)
					in0=n0/2; dn0=ceiling(0.25*n0)
					do
						call setlamk(in0)
						pwer=sum(boole(opttest)*densalt)/nsim
						if(pwer>.5) then 
							in0=min(in0+dn0,n0)
						else
							in0=max(1,in0-dn0)
						endif
						if(dn0==1) exit
						dn0=ceiling(0.5*dn0)
					enddo
					call setlamk(in0)
					pwer=sum(boole(opttest)*densalt)/nsim
					if(pwer>.5) in0=min(in0+1,n0)
					if(in0==n0) print *,"warning: optimal kappahat* outside grid"
					call setlamk(in0)
					pwer=sum(boole(opttest)*densalt)/nsim
					if(in0>1) then
						call setlamk(in0-1)
						pwer2=sum(boole(opttest)*densalt)/nsim
						outtab(2,ir,ikmax)=d0list(in0)-(d0list(in0)-d0list(in0-1))*(pwer2-.5)/(pwer-pwer2)
					else
						outtab(2,ir,ikmax)=0
					endif						
					in0=n0/2; dn0=ceiling(0.25*n0)
					do
						kappa=d0list(in0)
						call setLRtestk(in0)
						pwer=sum(boole(LRtest)*densalt)/nsim
						if(pwer>.5) then 
							in0=min(in0+dn0,n0)
						else
							in0=max(1,in0-dn0)
						endif
						if(dn0==1) exit
						dn0=ceiling(0.5*dn0)
					enddo
					kappa=d0list(in0)
					call setLRtestk(in0)
					pwer=sum(boole(LRtest)*densalt)/nsim
					if(pwer>.5) in0=min(in0+1,n0)
					kappa=d0list(in0)
					call setLRtestk(in0)
					pwer=sum(boole(LRtest)*densalt)/nsim

					if(in0>1) then
						kappa=d0list(in0-1)
						call setLRtestk(in0-1)
						pwer2=sum(boole(LRtest)*densalt)/nsim
						if(pwer2<=pwer) then 
							outtab(1,ir,ikmax)=d0list(in0)
						else
							outtab(1,ir,ikmax)=d0list(in0)-(d0list(in0)-d0list(in0-1))*(pwer2-0.5)/(pwer-pwer2)
						endif
					else
						outtab(1,ir,ikmax)=0
					endif						
				enddo
			enddo
			if(is==0) then
				print *,"k*, beta=0"
				call mdisp(outtab(1,:,:).clr.outtab(2,:,:))
			else
				print *,"k*, beta~N(0,5)"
				call mdisp(outtab(1,:,:).clr.outtab(2,:,:))
			endif
        enddo
	end subroutine

	subroutine setlamk(n0)
		integer	:: n0
		real	:: RP(n0), lamstep
		real	:: oldRP(n0), lamfac(n0), cv
		integer	:: l,j

		lamfac=4
		lam=1
		oldRP=level
		do j=1,500
			RP=0.0
!$omp parallel do reduction(+:RP)
			do l=1,nsim
				if(dot_product(dens0(1:n0,l),lam(1:n0))<densalt(l)) RP=RP+dens0(1:n0,l)
			enddo
			RP=RP/nsim
			lam(1:n0)=lam(1:n0)*exp(lamfac*(RP-level))
			where((RP-level)*(oldRP-level)>0)
				lamfac=lamfac*1.03
			elsewhere
				lamfac=lamfac/2.0
			endwhere
			oldRP=RP
			if(j>400) lamfac=0.1
			call setbounds(lamfac,0.01,20.0)
		enddo		
		opttest=matmul(lam(1:n0),dens0(1:n0,:))<densalt
	end subroutine

	subroutine setLRtestk(n0)
		integer	:: n0
		real	:: stats(nsim)
		integer	:: l,j
		real	:: cvs(n0)
		if(useAKtest) then
!$omp parallel do 
			do l=1,nsim
				LRtest(l)=getAKtest(Ys(:,l))
			enddo
			return
		endif

!$omp parallel do 
		do l=1,nsim
			stats(l)=getLRstat(Ys(:,l))
		enddo
!$omp parallel do 
		do j=1,n0
			cvs(j)=getcv(stats,dens0(j,:),level)
		enddo
		LRtest=stats>maxval(cvs)
	end subroutine
	
end module
	
program mfort
	use globals
	use compute
	implicit none

	call inittime
    call rnset(14)
	allocate(Ys(nY,nsim),densimp(nsim),dens0(n0,nsim),densalt(nsim),dens1(n0,nsim),LRtest(nsim),opttest(nsim))
	mmtesttab=loadmat(trim(dir)//"mtesttab.txt")
	
	useAKtest=.false.
	call comp
	call comp_minmax
	call lengthkappa
	call MSEcomp
	print *,"now switching to AK test and estimator"
	useAKtest=.true.
	call comp
	call comp_minmax
	call lengthkappa
	call MSEcomp	
end program